/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.sys.modular.basePigpen.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.sys.modular.basePigpen.entity.SysBasePigpen;
import vip.xiaonuo.sys.modular.basePigpen.mapper.SysBasePigpenMapper;
import vip.xiaonuo.sys.modular.basePigpen.param.SysBasePigpenAddParam;
import vip.xiaonuo.sys.modular.basePigpen.param.SysBasePigpenEditParam;
import vip.xiaonuo.sys.modular.basePigpen.param.SysBasePigpenIdParam;
import vip.xiaonuo.sys.modular.basePigpen.param.SysBasePigpenPageParam;
import vip.xiaonuo.sys.modular.basePigpen.service.SysBasePigpenService;

import java.util.List;

/**
 * 栋舍Service接口实现类
 *
 * @author newspaper
 * @date  2024/01/24 19:15
 **/
@Service
public class SysBasePigpenServiceImpl extends ServiceImpl<SysBasePigpenMapper, SysBasePigpen> implements SysBasePigpenService {

    @Override
    public Page<SysBasePigpen> page(SysBasePigpenPageParam sysBasePigpenPageParam) {
        QueryWrapper<SysBasePigpen> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isAllNotEmpty(sysBasePigpenPageParam.getSortField(), sysBasePigpenPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(sysBasePigpenPageParam.getSortOrder());
            queryWrapper.orderBy(true, sysBasePigpenPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(sysBasePigpenPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(SysBasePigpen::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(SysBasePigpenAddParam sysBasePigpenAddParam) {
        SysBasePigpen sysBasePigpen = BeanUtil.toBean(sysBasePigpenAddParam, SysBasePigpen.class);
        this.save(sysBasePigpen);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(SysBasePigpenEditParam sysBasePigpenEditParam) {
        SysBasePigpen sysBasePigpen = this.queryEntity(sysBasePigpenEditParam.getId());
        BeanUtil.copyProperties(sysBasePigpenEditParam, sysBasePigpen);
        this.updateById(sysBasePigpen);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<SysBasePigpenIdParam> sysBasePigpenIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(sysBasePigpenIdParamList, SysBasePigpenIdParam::getId));
    }

    @Override
    public SysBasePigpen detail(SysBasePigpenIdParam sysBasePigpenIdParam) {
        return this.queryEntity(sysBasePigpenIdParam.getId());
    }

    @Override
    public SysBasePigpen queryEntity(String id) {
        SysBasePigpen sysBasePigpen = this.getById(id);
        if(ObjectUtil.isEmpty(sysBasePigpen)) {
            throw new CommonException("栋舍不存在，id值为：{}", id);
        }
        return sysBasePigpen;
    }
}
