package vip.xiaonuo.Timer;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.timer.CommonTimerTaskRunner;
import vip.xiaonuo.modular.ai.taskInspection.entity.TaskInspection;
import vip.xiaonuo.modular.ai.taskInspection.mapper.TaskInspectionMapper;
import vip.xiaonuo.modular.ai.warningRemind.entity.WarningRemind;
import vip.xiaonuo.modular.ai.warningRemind.mapper.WarningRemindMapper;
import vip.xiaonuo.modular.base.baseConfig.entity.BaseConfig;
import vip.xiaonuo.modular.base.baseConfig.mapper.BaseConfigMapper;
import vip.xiaonuo.modular.feed.feedingenv.entity.FeedingEnv;
import vip.xiaonuo.modular.feed.feedingenv.mapper.FeedingEnvMapper;
import vip.xiaonuo.modular.feed.feedingfat.entity.FeedingFat;
import vip.xiaonuo.modular.feed.feedingfat.mapper.FeedingFatMapper;
import vip.xiaonuo.modular.feed.feedinglist.entity.FeedingList;
import vip.xiaonuo.modular.feed.feedinglist.mapper.FeedingListMapper;
import vip.xiaonuo.modular.feed.feedingthreshold.entity.FeedingThreshold;
import vip.xiaonuo.modular.feed.feedingthreshold.mapper.FeedingThresholdMapper;
import vip.xiaonuo.modular.warning.warningInfo.entity.WarningInfo;
import vip.xiaonuo.modular.warning.warningInfo.mapper.WarningInfoMapper;
import vip.xiaonuo.modular.warning.warningManager.entity.WarningManager;
import vip.xiaonuo.modular.warning.warningManager.mapper.WarningManagerMapper;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;


@Slf4j
@Component
public class ProFeedingData implements CommonTimerTaskRunner {

    @Autowired
    private FeedingListMapper listMapper;
    @Autowired
    private FeedingFatMapper fatMapper;
    @Autowired
    private FeedingEnvMapper envMapper;
    @Autowired
    private BaseConfigMapper baseConfigMapper;
    @Autowired
    private FeedingThresholdMapper feedingThresholdMapper;
    @Autowired
    private WarningRemindMapper warningRemindMapper;
    @Autowired
    private WarningManagerMapper warningManagerMapper;
    @Autowired
    private WarningInfoMapper warningInfoMapper;
    @Autowired
    private TaskInspectionMapper taskInspectionMapper;

    @Override
    public void action() {
        QueryWrapper<FeedingEnv> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("org_id", "1543842934270394370").eq("device_status", 1);
        List<FeedingEnv> feedingEnvs = envMapper.selectList(queryWrapper);
        for (FeedingEnv feedingEnv : feedingEnvs) {
            String unitId = feedingEnv.getUnitId();
            String orgId = feedingEnv.getOrgId();
            Date now = new Date();
            String water = NumberUtils.getNumFloat(5.0, 20.0);
            String feeding = NumberUtils.getNumFloat(5.0, 15.0);
            String weight = NumberUtils.getNumFloat(60.0, 65.0);
            String tem = NumberUtils.getNumFloat(36.0, 38.0);
            String act = NumberUtils.getNum(8, 10, 0);

            FeedingFat fat = new FeedingFat();
            fat.setUnitId(unitId);
            fat.setUnitName(feedingEnv.getUnitName());
            fat.setEartagNum(feedingEnv.getEartagNum());
            fat.setCreateTime(now);
            fat.setUploadTime(now);
            fat.setDeviceCode(feedingEnv.getDeviceCode());
            fat.setOrgId(orgId);
            fat.setDeviceType(feedingEnv.getType());
            fat.setWaterValue(water);
            fat.setWeigthValue(weight);
            fat.setFeedAct(act);
            fat.setFeedValue(feeding);
            fat.setTemp(tem);
            fat.setDayNum("100");
            if ("1".equals(feedingEnv.getType())) {
                fat.setRank(NumberUtils.getNumFloat(0.1, 1.0));
            }
            fatMapper.insert(fat);
            FeedingList list = new FeedingList();
            list.setUnitId(unitId);
            list.setUnitName(feedingEnv.getUnitName());
            list.setCreateTime(now);
            list.setUploadTime(now);
            list.setOrgId(orgId);
            list.setType(feedingEnv.getType());
            list.setPigNum(feedingEnv.getPigNum());
            String feedValue = NumberUtils.getNumFloat(30.0, 40.0);
            list.setFeedValue(feeding);
            list.setMarginValue(feedValue);
            listMapper.insert(list);

            BaseConfig baseConfig = baseConfigMapper.selectOne(new QueryWrapper<BaseConfig>().lambda().eq(BaseConfig::getConfigName, "预警类型").eq(BaseConfig::getParentId, 0));
            BaseConfig feedWarning = baseConfigMapper.selectOne(new QueryWrapper<BaseConfig>().lambda().eq(BaseConfig::getConfigName, "饲喂预警").eq(BaseConfig::getParentId, baseConfig.getId()));

            WarningRemind warningRemind = warningRemindMapper.selectOne(new QueryWrapper<WarningRemind>().lambda()
                    .eq(WarningRemind::getWarningTypeId, feedWarning.getId())
                    .eq(WarningRemind::getOrgId, orgId).eq(WarningRemind::getUnitId, unitId));

            if (ObjectUtil.isNotEmpty(warningRemind)) {
                if (warningRemind.getTodayNoRemind() == 1) {
                    continue;
                }
            }
            BigDecimal waterValue = new BigDecimal(water);
            BigDecimal feedingValue = new BigDecimal(feeding);

            FeedingThreshold fatFhreshold = feedingThresholdMapper.selectOne(new QueryWrapper<FeedingThreshold>().lambda().eq(FeedingThreshold::getOrgId, orgId).eq(FeedingThreshold::getThresholdType, 1));
            if (ObjectUtil.isNotEmpty(fatFhreshold)) {
                BigDecimal feedLower, feedUpper, waterLower, waterUpper;
                if ("1".equals(feedingEnv.getType())) {
                    feedLower = new BigDecimal(fatFhreshold.getFatFeedLower());
                    feedUpper = new BigDecimal(fatFhreshold.getFatFeedUpper());
                    waterLower = new BigDecimal(fatFhreshold.getFatWaterLower());
                    waterUpper = new BigDecimal(fatFhreshold.getFatWaterUpper());
                } else {
                    feedLower = new BigDecimal(fatFhreshold.getFeedLower());
                    feedUpper = new BigDecimal(fatFhreshold.getFeedUpper());
                    waterLower = new BigDecimal(fatFhreshold.getWaterLower());
                    waterUpper = new BigDecimal(fatFhreshold.getWaterUpper());
                }

                if (feedingValue.compareTo(feedLower) < 0) {
                    WarningInfo warningInfo = new WarningInfo();
                    warningInfo.setWarningTypeId(feedWarning.getId());
                    warningInfo.setOrgId(orgId);
                    warningInfo.setUnitId(unitId);
                    warningInfo.setWarningTime(now);
                    warningInfo.setWarningContent("耳标号" + feedingEnv.getEartagNum() + "采食" + feedingValue + "Kg低于阈值");
                    WarningManager warningManager = warningManagerMapper.selectOne(new QueryWrapper<WarningManager>().eq("type", 2).eq("org_id", orgId));
                    if (ObjectUtil.isNotEmpty(warningManager)) {
                        warningInfo.setWarningNotifierId(warningManager.getSecondManagerId());
                    }
                    warningInfoMapper.insert(warningInfo);
                    this.generateInspection(warningManager, warningInfo, orgId);
                }
                if (feedingValue.compareTo(feedUpper) > 0) {
                    WarningInfo warningInfo = new WarningInfo();
                    warningInfo.setWarningTypeId(feedWarning.getId());
                    warningInfo.setOrgId(orgId);
                    warningInfo.setUnitId(unitId);
                    warningInfo.setWarningTime(now);
                    warningInfo.setWarningContent("耳标号" + feedingEnv.getEartagNum() + "采食" + feedingValue + "Kg高于阈值");
                    WarningManager warningManager = warningManagerMapper.selectOne(new QueryWrapper<WarningManager>().eq("type", 2).eq("org_id", orgId));
                    if (ObjectUtil.isNotEmpty(warningManager)) {
                        warningInfo.setWarningNotifierId(warningManager.getSecondManagerId());
                    }
                    warningInfoMapper.insert(warningInfo);
                    this.generateInspection(warningManager, warningInfo, orgId);
                }
                if (waterValue.compareTo(waterLower) < 0) {
                    WarningInfo warningInfo = new WarningInfo();
                    warningInfo.setWarningTypeId(feedWarning.getId());
                    warningInfo.setOrgId(orgId);
                    warningInfo.setUnitId(unitId);
                    warningInfo.setWarningTime(now);
                    warningInfo.setWarningContent("耳标号" + feedingEnv.getEartagNum() + "饮水" + waterValue + "Kg低于阈值");
                    WarningManager warningManager = warningManagerMapper.selectOne(new QueryWrapper<WarningManager>().eq("type", 2).eq("org_id", orgId));
                    if (ObjectUtil.isNotEmpty(warningManager)) {
                        warningInfo.setWarningNotifierId(warningManager.getSecondManagerId());
                    }
                    warningInfoMapper.insert(warningInfo);
                    this.generateInspection(warningManager, warningInfo, orgId);
                }
                if (waterValue.compareTo(waterUpper) > 0) {
                    WarningInfo warningInfo = new WarningInfo();
                    warningInfo.setWarningTypeId(feedWarning.getId());
                    warningInfo.setOrgId(orgId);
                    warningInfo.setUnitId(unitId);
                    warningInfo.setWarningTime(now);
                    warningInfo.setWarningContent("耳标号" + feedingEnv.getEartagNum() + "饮水" + waterValue + "Kg高于阈值");
                    WarningManager warningManager = warningManagerMapper.selectOne(new QueryWrapper<WarningManager>().eq("type", 2).eq("org_id", orgId));
                    if (ObjectUtil.isNotEmpty(warningManager)) {
                        warningInfo.setWarningNotifierId(warningManager.getSecondManagerId());
                    }
                    warningInfoMapper.insert(warningInfo);
                    this.generateInspection(warningManager, warningInfo, orgId);
                }
            }

            FeedingThreshold feedingThreshold = feedingThresholdMapper.selectOne(new QueryWrapper<FeedingThreshold>().lambda().eq(FeedingThreshold::getFeedingUnitId, unitId).eq(FeedingThreshold::getOrgId, orgId));
            if (ObjectUtil.isNotEmpty(feedingThreshold)) {
                BigDecimal marginValue = new BigDecimal(feedingThreshold.getMarginValue());
                BigDecimal marginFeedValue = new BigDecimal(feedValue);
                if (marginFeedValue.compareTo(marginValue) < 0) {
                    WarningInfo warningInfo = new WarningInfo();
                    warningInfo.setWarningTypeId(feedWarning.getId());
                    warningInfo.setOrgId(orgId);
                    warningInfo.setUnitId(unitId);
                    warningInfo.setWarningTime(now);
                    warningInfo.setWarningContent(feedingEnv.getUnitName() + "余料" + marginFeedValue + "吨低于阈值");
                    WarningManager warningManager = warningManagerMapper.selectOne(new QueryWrapper<WarningManager>().eq("type", 2).eq("org_id", orgId));
                    if (ObjectUtil.isNotEmpty(warningManager)) {
                        warningInfo.setWarningNotifierId(warningManager.getSecondManagerId());
                    }
                    warningInfoMapper.insert(warningInfo);
                    this.generateInspection(warningManager, warningInfo, orgId);
                }
            }


        }
    }

    private void generateInspection(WarningManager warningManager, WarningInfo warningInfo, String orgId) {
        if (ObjectUtil.isNotEmpty(warningManager)) {
            TaskInspection taskInspection = new TaskInspection();
            taskInspection.setNotifierId(warningManager.getSecondManagerId());
            taskInspection.setWarningId(warningInfo.getId());
            taskInspection.setOrgId(orgId);
            taskInspectionMapper.insert(taskInspection);
        }
    }
}
