/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.ai.carCapture.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.auth.core.util.StpLoginUserUtil;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.core.param.OrgIdParam;
import vip.xiaonuo.modular.ai.carCapture.entity.CarCapture;
import vip.xiaonuo.modular.ai.carCapture.entity.vo.CarCaptureHourVo;
import vip.xiaonuo.modular.ai.carCapture.entity.vo.CarCapturePastVo;
import vip.xiaonuo.modular.ai.carCapture.mapper.CarCaptureMapper;
import vip.xiaonuo.modular.ai.carCapture.param.CarCaptureAddParam;
import vip.xiaonuo.modular.ai.carCapture.param.CarCaptureEditParam;
import vip.xiaonuo.modular.ai.carCapture.param.CarCaptureIdParam;
import vip.xiaonuo.modular.ai.carCapture.param.CarCapturePageParam;
import vip.xiaonuo.modular.ai.carCapture.service.CarCaptureService;
import vip.xiaonuo.modular.ai.followEvents.entity.FollowEvents;
import vip.xiaonuo.modular.ai.followEvents.service.FollowEventsService;
import vip.xiaonuo.modular.ai.personCapture.entity.PersonCapture;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.List;

/**
 * 车辆识别Service接口实现类
 *
 * @author newspaper
 * @date  2023/12/29 09:42
 **/
@Service
public class CarCaptureServiceImpl extends ServiceImpl<CarCaptureMapper, CarCapture> implements CarCaptureService {
    @Autowired
    private CarCaptureMapper carCaptureMapper;
    @Autowired
    private FollowEventsService followEventsService;

    @Override
    public Page<CarCapture> page(CarCapturePageParam carCapturePageParam) {
        String orgId = carCapturePageParam.getOrgId();
        QueryWrapper<CarCapture> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CarCapture::getOrgId,orgId).orderByDesc(CarCapture::getCapTime);
        if(ObjectUtil.isNotEmpty(carCapturePageParam.getStartCapTime()) && ObjectUtil.isNotEmpty(carCapturePageParam.getEndCapTime())) {
            String startTime = carCapturePageParam.getStartCapTime() + " 00:00:00";
            String endTime = carCapturePageParam.getEndCapTime() + " 23:59:59";
            queryWrapper.lambda().between(CarCapture::getCapTime, startTime, endTime);
        }
        if(ObjectUtil.isNotEmpty(carCapturePageParam.getCarType())) {
            queryWrapper.lambda().eq(CarCapture::getCarType, carCapturePageParam.getCarType());
        }
        if(ObjectUtil.isNotEmpty(carCapturePageParam.getCapResult())) {
            queryWrapper.lambda().eq(CarCapture::getCapResult, carCapturePageParam.getCapResult());
        }
        Page<CarCapture> page = this.page(CommonPageRequest.defaultPage(), queryWrapper);
        List<CarCapture> records = page.getRecords();
        for (CarCapture record : records) {
            long count = followEventsService.count(new QueryWrapper<FollowEvents>().lambda()
                    .eq(FollowEvents::getEventId, record.getId())
                    .eq(FollowEvents::getEventType, 2).eq(FollowEvents::getUserId, StpLoginUserUtil.getLoginUser().getId()));
            if (count > 0){
                record.setIsFollow(1);
            }else {
                record.setIsFollow(0);
            }
        }
        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(CarCaptureAddParam carCaptureAddParam) {
        CarCapture carCapture = BeanUtil.toBean(carCaptureAddParam, CarCapture.class);
        this.save(carCapture);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(CarCaptureEditParam carCaptureEditParam) {
        CarCapture carCapture = this.queryEntity(carCaptureEditParam.getId());
        BeanUtil.copyProperties(carCaptureEditParam, carCapture);
        this.updateById(carCapture);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<CarCaptureIdParam> carCaptureIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(carCaptureIdParamList, CarCaptureIdParam::getId));
    }

    @Override
    public CarCapture detail(CarCaptureIdParam carCaptureIdParam) {
        return this.queryEntity(carCaptureIdParam.getId());
    }

    @Override
    public CarCapture queryEntity(String id) {
        CarCapture carCapture = this.getById(id);
        if(ObjectUtil.isEmpty(carCapture)) {
            throw new CommonException("车辆识别不存在，id值为：{}", id);
        }
        return carCapture;
    }

    @Override
    public CarCapturePastVo getPastCapture(OrgIdParam orgIdParam) {
        CarCapturePastVo carCapturePastVo = new CarCapturePastVo();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        // 获取当前时间
        LocalDateTime currentTime = LocalDateTime.now();
        // 计算12小时前的时间
        LocalDateTime twelveHoursAgo = currentTime.minusHours(12);
        String twelveHoursAgoFormatted = twelveHoursAgo.format(formatter);
        long count = this.count(new QueryWrapper<CarCapture>().lambda().eq(CarCapture::getOrgId, orgIdParam.getOrgId()).ge(CarCapture::getCapTime, twelveHoursAgoFormatted));
        carCapturePastVo.setTotal(count);
        List<CarCaptureHourVo> carCaptureHourVos = carCaptureMapper.listPast(orgIdParam.getOrgId());
        carCapturePastVo.setList(carCaptureHourVos);
        return carCapturePastVo;
    }
}
