/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.ai.followEvents.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.modular.ai.followEvents.entity.FollowEvents;
import vip.xiaonuo.modular.ai.followEvents.entity.FollowEventsVo;
import vip.xiaonuo.modular.ai.followEvents.param.FollowEventsAddParam;
import vip.xiaonuo.modular.ai.followEvents.param.FollowEventsEditParam;
import vip.xiaonuo.modular.ai.followEvents.param.FollowEventsIdParam;
import vip.xiaonuo.modular.ai.followEvents.param.FollowEventsPageParam;
import vip.xiaonuo.modular.ai.followEvents.service.FollowEventsService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 事件关注控制器
 *
 * @author newspaper
 * @date  2024/01/24 10:50
 */
@Api(tags = "事件关注控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class FollowEventsController {

    @Resource
    private FollowEventsService followEventsService;

    /**
     * 获取事件关注分页
     *
     * @author newspaper
     * @date  2024/01/24 10:50
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取事件关注分页")
    @GetMapping("/ai/followEvents/page")
    public CommonResult<Page<FollowEventsVo>> page(FollowEventsPageParam followEventsPageParam) {
        return CommonResult.data(followEventsService.page(followEventsPageParam));
    }

    /**
     * 添加事件关注
     *
     * @author newspaper
     * @date  2024/01/24 10:50
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加事件关注")
    @CommonLog("添加事件关注")
    @PostMapping("/ai/followEvents/add")
    public CommonResult<String> add(@RequestBody @Valid FollowEventsAddParam followEventsAddParam) {
        followEventsService.add(followEventsAddParam);
        return CommonResult.ok("关注成功");
    }

    /**
     * 编辑事件关注
     *
     * @author newspaper
     * @date  2024/01/24 10:50
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑事件关注")
    @CommonLog("编辑事件关注")
    @PostMapping("/ai/followEvents/edit")
    public CommonResult<String> edit(@RequestBody @Valid FollowEventsEditParam followEventsEditParam) {
        followEventsService.edit(followEventsEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除事件关注
     *
     * @author newspaper
     * @date  2024/01/24 10:50
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除事件关注")
    @CommonLog("删除事件关注")
    @PostMapping("/ai/followEvents/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<FollowEventsIdParam> followEventsIdParamList) {
        followEventsService.delete(followEventsIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取事件关注详情
     *
     * @author newspaper
     * @date  2024/01/24 10:50
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取事件关注详情")
    @GetMapping("/ai/followEvents/detail")
    public CommonResult<FollowEvents> detail(@Valid FollowEventsIdParam followEventsIdParam) {
        return CommonResult.data(followEventsService.detail(followEventsIdParam));
    }


    /**
     * 单独事件取消关注
     *
     * @author newspaper
     * @date  2024/01/24 10:50
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("单独事件取消关注")
    @PostMapping("/ai/followEvents/cancel")
    public CommonResult<String> cancel(@RequestBody FollowEventsAddParam followEventsAddParam) {
        followEventsService.cancel(followEventsAddParam);
        return new CommonResult<>(200,"取消关注成功","取消关注成功");
    }
}
