/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.ai.speciesCapture.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.modular.ai.speciesCapture.entity.SpeciesCapture;
import vip.xiaonuo.modular.ai.speciesCapture.param.SpeciesCaptureAddParam;
import vip.xiaonuo.modular.ai.speciesCapture.param.SpeciesCaptureEditParam;
import vip.xiaonuo.modular.ai.speciesCapture.param.SpeciesCaptureIdParam;
import vip.xiaonuo.modular.ai.speciesCapture.param.SpeciesCapturePageParam;
import vip.xiaonuo.modular.ai.speciesCapture.service.SpeciesCaptureService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 外来生物识别控制器
 *
 * @author newspaper
 * @date  2024/01/06 11:32
 */
@Api(tags = "外来生物识别控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class SpeciesCaptureController {

    @Resource
    private SpeciesCaptureService speciesCaptureService;

    /**
     * 获取外来生物识别分页
     *
     * @author newspaper
     * @date  2024/01/06 11:32
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取外来生物识别分页")
    @GetMapping("/ai/speciesCapture/page")
    public CommonResult<Page<SpeciesCapture>> page(SpeciesCapturePageParam speciesCapturePageParam) {
        return CommonResult.data(speciesCaptureService.page(speciesCapturePageParam));
    }

    /**
     * 添加外来生物识别
     *
     * @author newspaper
     * @date  2024/01/06 11:32
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加外来生物识别")
    @CommonLog("添加外来生物识别")
    @PostMapping("/ai/speciesCapture/add")
    public CommonResult<String> add(@RequestBody @Valid SpeciesCaptureAddParam speciesCaptureAddParam) {
        speciesCaptureService.add(speciesCaptureAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑外来生物识别
     *
     * @author newspaper
     * @date  2024/01/06 11:32
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑外来生物识别")
    @CommonLog("编辑外来生物识别")
    @PostMapping("/ai/speciesCapture/edit")
    public CommonResult<String> edit(@RequestBody @Valid SpeciesCaptureEditParam speciesCaptureEditParam) {
        speciesCaptureService.edit(speciesCaptureEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除外来生物识别
     *
     * @author newspaper
     * @date  2024/01/06 11:32
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除外来生物识别")
    @CommonLog("删除外来生物识别")
    @PostMapping("/ai/speciesCapture/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<SpeciesCaptureIdParam> speciesCaptureIdParamList) {
        speciesCaptureService.delete(speciesCaptureIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取外来生物识别详情
     *
     * @author newspaper
     * @date  2024/01/06 11:32
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取外来生物识别详情")
    @GetMapping("/ai/speciesCapture/detail")
    public CommonResult<SpeciesCapture> detail(@Valid SpeciesCaptureIdParam speciesCaptureIdParam) {
        return CommonResult.data(speciesCaptureService.detail(speciesCaptureIdParam));
    }
}
