/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.base.inventoryUse.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import vip.xiaonuo.auth.core.util.StpLoginUserUtil;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.modular.base.baseConfig.entity.BaseConfig;
import vip.xiaonuo.modular.base.baseConfig.service.BaseConfigService;
import vip.xiaonuo.modular.base.department.service.DepartmentService;
import vip.xiaonuo.modular.base.inventoryUse.entity.InventoryUse;
import vip.xiaonuo.modular.base.inventoryUse.entity.vo.InventoryUseDetailVo;
import vip.xiaonuo.modular.base.inventoryUse.entity.vo.InventoryUseVo;
import vip.xiaonuo.modular.base.inventoryUse.param.*;
import vip.xiaonuo.modular.base.inventoryUse.service.InventoryUseService;
import vip.xiaonuo.modular.base.peoplelist.entity.PeopleList;
import vip.xiaonuo.modular.base.peoplelist.service.PeopleListService;
import vip.xiaonuo.sys.modular.user.entity.SysUser;
import vip.xiaonuo.sys.modular.user.service.SysUserService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.io.IOException;
import java.text.ParseException;
import java.util.Date;
import java.util.List;

/**
 * 物资领用控制器
 *
 * @author newspaper
 * @date  2023/12/11 10:43
 */
@Api(tags = "物资领用控制器")
@ApiSupport(author = "HUIMV", order = 1)
@RestController
@Validated
public class InventoryUseController {

    @Resource
    private InventoryUseService inventoryUseService;
    @Resource
    private BaseConfigService baseConfigService;
    @Resource
    private SysUserService sysUserService;

    /**
     * 获取物资领用分页
     *
     * @author newspaper
     * @date  2023/12/11 10:43
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取物资领用分页")
    @GetMapping("/base/inventoryUse/page")
    public CommonResult<Page<InventoryUse>> page(InventoryUsePageParam inventoryUsePageParam) {
        return CommonResult.data(inventoryUseService.page(inventoryUsePageParam));
    }

    /**
     * 添加物资领用
     *
     * @author newspaper
     * @date  2023/12/11 10:43
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加物资领用")
    @CommonLog("添加物资领用")
    @PostMapping("/base/inventoryUse/add")
    public CommonResult<String> add(@RequestBody @Valid InventoryUseAddParam inventoryUseAddParam) {
        inventoryUseService.add(inventoryUseAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑物资领用
     *
     * @author newspaper
     * @date  2023/12/11 10:43
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑物资领用")
    @CommonLog("编辑物资领用")
    @PostMapping("/base/inventoryUse/edit")
    public CommonResult<String> edit(@RequestBody @Valid InventoryUseEditParam inventoryUseEditParam) {
        inventoryUseService.edit(inventoryUseEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除物资领用
     *
     * @author newspaper
     * @date  2023/12/11 10:43
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除物资领用")
    @CommonLog("删除物资领用")
    @PostMapping("/base/inventoryUse/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<InventoryUseIdParam> inventoryUseIdParamList) {
        inventoryUseService.delete(inventoryUseIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取物资领用详情
     *
     * @author newspaper
     * @date  2023/12/11 10:43
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取物资领用详情")
    @GetMapping("/base/inventoryUse/detail")
    public CommonResult<InventoryUse> detail(@Valid InventoryUseIdParam inventoryUseIdParam) {
        return CommonResult.data(inventoryUseService.detail(inventoryUseIdParam));
    }

    /**
     * 编辑物资领用
     *
     * @author newspaper
     * @date  2023/12/11 10:43
     */
    @ApiOperationSupport(order = 6)
    @ApiOperation("归还物资领用")
    @CommonLog("归还物资领用")
    @PostMapping("/base/inventoryUse/revert")
    public CommonResult<String> revert(@RequestParam MultipartFile returnImgUrl,
                                       @RequestParam String id,
                                       @RequestParam String returnTime,
                                       @RequestParam String remarks) throws ParseException, IOException {
        inventoryUseService.revert(returnImgUrl,id,returnTime,remarks);
        return CommonResult.ok();
    }

    /**
     * 获取物资类别选择器
     *
     * @author newspaper
     * @date  2023/12/11 10:43
     */
    @ApiOperationSupport(order = 7)
    @ApiOperation("获取物资类别选择器")
    @GetMapping("/base/inventoryUse/typeSelector")
    public CommonResult<List<BaseConfig>> typeSelector(InventoryUseOrgIdParam inventoryUseOrgIdParam) {
        String orgId = inventoryUseOrgIdParam.getOrgId();
        BaseConfig baseConfig = baseConfigService.getOne(new QueryWrapper<BaseConfig>().lambda().eq(BaseConfig::getConfigName, "物资类别").eq(BaseConfig::getParentId, 0));
        List<BaseConfig> baseConfigs = baseConfigService.list(new QueryWrapper<BaseConfig>().lambda().eq(BaseConfig::getParentId, baseConfig.getId()).eq(BaseConfig::getStatus,1).eq(BaseConfig::getOrgId, orgId));
        return CommonResult.data(baseConfigs);
    }

    /**
     * 获取仓库选择器
     *
     * @author newspaper
     * @date  2023/12/11 10:43
     */
    @ApiOperationSupport(order = 8)
    @ApiOperation("获取仓库选择器")
    @GetMapping("/base/inventoryUse/warehouseSelector")
    public CommonResult<List<BaseConfig>> warehouseSelector(InventoryUseOrgIdParam inventoryUseOrgIdParam) {
        String orgId = inventoryUseOrgIdParam.getOrgId();
        BaseConfig baseConfig = baseConfigService.getOne(new QueryWrapper<BaseConfig>().lambda().eq(BaseConfig::getConfigName, "仓库").eq(BaseConfig::getParentId, 0));
        List<BaseConfig> baseConfigs = baseConfigService.list(new QueryWrapper<BaseConfig>().lambda().eq(BaseConfig::getStatus,1).eq(BaseConfig::getParentId, baseConfig.getId()).eq(BaseConfig::getOrgId, orgId));
        return CommonResult.data(baseConfigs);
    }

    /**
     * 获取部门人员选择器
     *
     * @author newspaper
     * @date  2023/12/11 10:43
     */
    @ApiOperationSupport(order = 9)
    @ApiOperation("获取部门人员选择器")
    @GetMapping("/base/inventoryUse/departmentPersonSelector")
    public CommonResult<List<SysUser>> departmentPersonSelector(InventoryUseDepartmentIdParam inventoryUseDepartmentIdParam) {
        List<SysUser> list = sysUserService.list(new QueryWrapper<SysUser>().lambda().eq(SysUser::getDepartmentId, inventoryUseDepartmentIdParam.getDepartmentId()));
        return CommonResult.data(list);
    }
}
